==> sources_fancymenu_fabric_3.8.0_MC_1.19.2.jar: de/keksuccino/fancymenu/util/rendering/ui/tooltip/Tooltip.java <==
package de.keksuccino.fancymenu.util.rendering.ui.tooltip;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.FancyMenu;
import de.keksuccino.fancymenu.util.rendering.DrawableColor;
import de.keksuccino.fancymenu.util.rendering.RenderingUtils;
import de.keksuccino.fancymenu.util.rendering.gui.GuiGraphics;
import de.keksuccino.fancymenu.util.rendering.gui.Renderable;
import de.keksuccino.fancymenu.util.rendering.gui.TooltipRenderUtil;
import de.keksuccino.fancymenu.util.rendering.text.TextFormattingUtils;
import de.keksuccino.fancymenu.util.rendering.ui.UIBase;
import de.keksuccino.fancymenu.util.resource.resources.texture.ITexture;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import net.minecraft.class_2561;
import net.minecraft.class_2583;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_437;
import net.minecraft.class_5250;
import net.minecraft.class_5348;

/**
 * A tooltip that gets rendered at the mouse position by default.<br>
 * It is possible to set a custom X and Y position to not render it at the mouse position.
 **/
@SuppressWarnings("unused")
public class Tooltip implements Renderable {

    private static final Logger LOGGER = LogManager.getLogger();

    protected class_327 font = class_310.method_1551().field_1772;
    protected List<class_2561> textLines = new ArrayList<>();
    protected int width = 0;
    protected int height = 0;
    protected int aspectWidth = -1;
    protected int aspectHeight = -1;
    protected Integer x = null;
    protected Integer y = null;
    protected int textBorderSize = 5;
    protected int mouseOffset = 12;
    protected ITexture backgroundTexture = null;
    protected DrawableColor backgroundColor = null;
    protected DrawableColor borderColor = null;
    protected boolean vanillaLike = true;
    protected boolean keepBackgroundAspectRatio = true;
    protected boolean textShadow = true;
    protected TooltipTextAlignment textAlignment = TooltipTextAlignment.LEFT;
    protected DrawableColor textBaseColor = null;
    protected Float scale = null;

    @NotNull
    public static Tooltip empty() {
        return new Tooltip();
    }

    @NotNull
    public static Tooltip of(String... tooltip) {
        Tooltip t = new Tooltip();
        if (tooltip != null) {
            t.setTooltipText(tooltip);
        }
        return t;
    }

    @NotNull
    public static Tooltip of(class_2561... tooltip) {
        Tooltip t = new Tooltip();
        if (tooltip != null) {
            t.setTooltipText(tooltip);
        }
        return t;
    }

    @Override
    public void render(@NotNull GuiGraphics graphics, int mouseX, int mouseY, float partial) {

        if (RenderingUtils.isTooltipRenderingBlocked()) return;

        class_437 s = class_310.method_1551().field_1755;
        if (!this.isEmpty() && (s != null)) {

            //Dummy-render a Vanilla tooltip to fix the custom one (this is hacky af and I hate it, but I don't have time to analyze the whole frickin Vanilla tooltip rendering tbh)
            graphics.renderTooltip(class_310.method_1551().field_1772, class_2561.method_43473(), -10000, -10000);

            this.updateAspectRatio();

            int x = this.calculateX(s, mouseX);
            int y = this.calculateY(s, mouseY);

            RenderSystem.enableBlend();

            graphics.pose().method_22903();

            float scale = 1.0F;
            if (this.scale != null) {
                scale = UIBase.calculateFixedScale(this.scale);
                graphics.pose().method_22905(scale, scale, scale);
            }
            graphics.pose().method_22904(0.0F, 0.0F, 600.0F / scale);
            RenderSystem.enableDepthTest();
            RenderingUtils.resetShaderColor(graphics);

            this.renderBackground(graphics, x, y);
            this.renderTextLines(graphics, x, y);

            RenderSystem.disableDepthTest();
            graphics.pose().method_22909();

            RenderingUtils.resetShaderColor(graphics);

        }
    }

    protected void renderTextLines(GuiGraphics graphics, int x, int y) {
        int yLine = y;
        for (class_2561 c : this.textLines) {
            int w = this.font.method_27525(c);
            int x2 = x + this.textBorderSize;
            int y2 = yLine + this.textBorderSize;
            if (this.textAlignment == TooltipTextAlignment.RIGHT) {
                int diff = Math.max(0, (x + this.getWidth() - this.textBorderSize) - (x2 + w));
                x2 += diff;
            }
            if (this.textAlignment == TooltipTextAlignment.CENTERED) {
                x2 = x + Math.max(0, (this.getWidth() / 2) - (w / 2));
            }
            graphics.drawString(this.font, c, x2, y2, (this.textBaseColor != null) ? this.textBaseColor.getColorInt() : -1, this.hasTextShadow());
            yLine += this.font.field_2000 + 2;
        }
    }

    protected void renderBackground(GuiGraphics graphics, int x, int y) {
        if (this.vanillaLike || ((this.backgroundTexture == null) && (this.backgroundColor == null))) {
            this.renderVanillaLikeBackground(graphics, x, y, this.getWidth(), this.getHeight());
        } else if (this.backgroundTexture != null) {
            class_2960 loc = this.backgroundTexture.getResourceLocation();
            if (loc != null) {
                graphics.blit(loc, x, y, 0.0F, 0.0F, this.getWidth(), this.getHeight(), this.getWidth(), this.getHeight());
            }
        } else {
            if (this.borderColor != null) {
                //BACKGROUND
                graphics.fill(x + 1, y + 1, x + this.getWidth() - 1, y + this.getHeight() - 1, this.backgroundColor.getColorInt());
                //TOP
                graphics.fill(x + 1, y, x + this.getWidth() - 1, y + 1, this.borderColor.getColorInt());
                //BOTTOM
                graphics.fill(x + 1, y + this.getHeight() - 1, x + this.getWidth() - 1, y + this.getHeight(), this.borderColor.getColorInt());
                //LEFT
                graphics.fill(x, y, x + 1, y + this.getHeight(), this.borderColor.getColorInt());
                //RIGHT
                graphics.fill(x + this.getWidth() - 1, y, x + this.getWidth(), y + this.getHeight(), this.borderColor.getColorInt());
            } else {
                graphics.fill(x, y, x + this.getWidth(), y + this.getHeight(), this.backgroundColor.getColorInt());
            }
        }
    }

    protected void renderVanillaLikeBackground(GuiGraphics graphics, int x, int y, int width, int height) {

        graphics.pose().method_22903();

        //Set Z to 0, because Z level gets handled in parent method instead
        int z = 0;
        graphics.drawManaged(() -> TooltipRenderUtil.renderTooltipBackground(graphics, x, y, width, height, z));

        graphics.pose().method_22909();

    }

    protected int calculateX(class_437 screen, int mouseX) {
        float scale = 1;
        if (this.scale != null) {
            scale = UIBase.calculateFixedScale(this.scale);
        }
        if (this.x != null) {
            mouseX = this.x;
        }
        int width = this.getWidth();
        if (this.vanillaLike) {
            width += 4;
        }
        int scaledWidth = (int)((float)width * scale);
        int scaledMouseX = (int)((float)mouseX / scale) + this.mouseOffset;
        int scaledScreenWidth = (int)((float)screen.field_22789 / scale);
        int x = (mouseX + this.mouseOffset);
        if ((x + scaledWidth) > screen.field_22789) {
            int offset = (x + scaledWidth) - screen.field_22789;
            x = x - offset;
        }
        return (int)((float)x / scale);
    }

    protected int calculateY(class_437 screen, int mouseY) {
        float scale = 1;
        if (this.scale != null) {
            scale = UIBase.calculateFixedScale(this.scale);
        }
        if (this.y != null) {
            mouseY = this.y;
        }
        int height = this.getHeight();
        if (this.vanillaLike) {
            height += 4;
        }
        int scaledHeight = (int)((float)height * scale);
        int scaledMouseY = (int)((float)mouseY / scale) + this.mouseOffset;
        int scaledScreenHeight = (int)((float)screen.field_22790 / scale);
        int y = (mouseY + this.mouseOffset);
        if ((y + scaledHeight) > screen.field_22790) {
            int offset = (y + scaledHeight) - screen.field_22790;
            y = y - offset;
        }
        return (int)((float)y / scale);
    }

    protected int getWidth() {
        if (this.aspectWidth != -1) {
            return this.aspectWidth;
        }
        return this.width;
    }

    protected int getHeight() {
        if (this.aspectHeight != -1) {
            return this.aspectHeight;
        }
        return this.height;
    }

    protected void updateAspectRatio() {
        if (!this.keepBackgroundAspectRatio || (this.backgroundTexture == null)) {
            this.aspectWidth = -1;
            this.aspectHeight = -1;
            return;
        }
        int[] size = this.backgroundTexture.getAspectRatio().getAspectRatioSizeByMinimumSize(this.width, this.height);
        this.aspectWidth = size[0];
        this.aspectHeight = size[1];
    }

    protected void updateSize() {
        int w = 0;
        int h = 0;
        for (class_2561 c : this.textLines) {
            int wl = this.font.method_27525(c);
            if (wl > w) {
                w = wl;
            }
            h += this.font.field_2000 + 2;
        }
        if (h > 0) {
            h -= 2;
        }
        this.width = w + (this.textBorderSize * 2);
        this.height = h + (this.textBorderSize * 2);
    }

    public boolean isEmpty() {
        return this.textLines.isEmpty();
    }

    public Tooltip setTooltipText(String... lines) {
        List<class_2561> l = new ArrayList<>();
        if (lines != null) {
            for (String s : lines) {
                l.add(class_2561.method_43470(s));
            }
        }
        return this.setTooltipText(l);
    }

    public Tooltip setTooltipText(class_2561... lines) {
        return this.setTooltipText((lines != null) ? Arrays.asList(lines) : null);
    }

    public Tooltip setTooltipText(List<class_2561> lines) {
        class_5250 merged = class_2561.method_43473();
        if (lines != null) {
            boolean b = true;
            for (class_2561 c : lines) {
                if (!b) merged.method_27693("\n");
                merged.method_10852(c);
                b = false;
            }
        }
        this.textLines = new ArrayList<>();
        for (class_5348 s : class_310.method_1551().field_1772.method_27527().method_27495(merged, 300, class_2583.field_24360)) {
            this.textLines.add(TextFormattingUtils.convertFormattedTextToComponent(s));
        }
        this.updateSize();
        return this;
    }

    /** Returns a COPY of the tooltip list. **/
    public List<class_2561> getTooltip() {
        return new ArrayList<>(this.textLines);
    }

    public Tooltip setTextBorderSize(int size) {
        this.textBorderSize = size;
        this.updateSize();
        return this;
    }

    public int getTextBorderSize() {
        return this.textBorderSize;
    }

    public Tooltip setMouseOffset(int offset) {
        this.mouseOffset = offset;
        return this;
    }

    public int getMouseOffset() {
        return this.mouseOffset;
    }

    public Tooltip setBackgroundTexture(@NotNull ITexture texture) {
        this.backgroundTexture = texture;
        this.backgroundColor = null;
        this.vanillaLike = false;
        return this;
    }

    @Nullable
    public ITexture getBackgroundTexture() {
        return this.backgroundTexture;
    }

    public Tooltip setBackgroundColor(@NotNull DrawableColor backgroundColor, @Nullable DrawableColor borderColor) {
        this.backgroundColor = backgroundColor;
        this.borderColor = borderColor;
        this.backgroundTexture = null;
        this.vanillaLike = false;
        return this;
    }

    public Tooltip setDefaultStyle() {
        this.setTextShadow(FancyMenu.getOptions().enableUiTextShadow.getValue());
        this.setTextBaseColor(UIBase.getUIColorTheme().element_label_color_normal);
        return this.setBackgroundColor(UIBase.getUIColorTheme().element_background_color_normal, UIBase.getUIColorTheme().element_border_color_normal);
    }

    @Nullable
    public DrawableColor getBackgroundColor() {
        return this.backgroundColor;
    }

    @Nullable
    public DrawableColor getBorderColor() {
        return this.borderColor;
    }

    public Tooltip setVanillaLike(boolean vanillaLike) {
        this.vanillaLike = vanillaLike;
        this.backgroundColor = null;
        this.backgroundTexture = null;
        return this;
    }

    public boolean isVanillaLike() {
        return vanillaLike;
    }

    public Tooltip setKeepBackgroundAspectRatio(boolean keepBackgroundAspectRatio) {
        this.keepBackgroundAspectRatio = keepBackgroundAspectRatio;
        return this;
    }

    public boolean keepBackgroundAspectRatio() {
        return keepBackgroundAspectRatio;
    }

    public Tooltip setTextShadow(boolean textShadow) {
        this.textShadow = textShadow;
        return this;
    }

    public boolean hasTextShadow() {
        return textShadow;
    }

    public Tooltip setTextAlignment(TooltipTextAlignment textAlignment) {
        this.textAlignment = textAlignment;
        return this;
    }

    public TooltipTextAlignment getTextAlignment() {
        return textAlignment;
    }

    public Tooltip setTextBaseColor(@Nullable DrawableColor textBaseColor) {
        this.textBaseColor = textBaseColor;
        return this;
    }

    @Nullable
    public DrawableColor getTextBaseColor() {
        return textBaseColor;
    }

    public Tooltip setFont(class_327 font) {
        this.font = (font != null) ? font : class_310.method_1551().field_1772;
        this.updateSize();
        return this;
    }

    public class_327 getFont() {
        return font;
    }

    public Tooltip copyStyleOf(Tooltip tooltip) {
        this.borderColor = tooltip.borderColor;
        this.backgroundColor = tooltip.backgroundColor;
        this.backgroundTexture = tooltip.backgroundTexture;
        this.keepBackgroundAspectRatio = tooltip.keepBackgroundAspectRatio;
        this.vanillaLike = tooltip.vanillaLike;
        this.textBorderSize = tooltip.textBorderSize;
        this.textAlignment = tooltip.textAlignment;
        this.textShadow = tooltip.textShadow;
        this.textBaseColor = tooltip.textBaseColor;
        this.font = tooltip.font;
        return this;
    }

    public Tooltip setCustomX(@Nullable Integer x) {
        this.x = x;
        return this;
    }

    @Nullable
    public Integer getCustomX() {
        return x;
    }

    public Tooltip setCustomY(@Nullable Integer y) {
        this.y = y;
        return this;
    }

    @Nullable
    public Integer getCustomY() {
        return y;
    }

    @Nullable
    public Float getScale() {
        return this.scale;
    }

    public Tooltip setScale(@Nullable Float scale) {
        this.scale = scale;
        return this;
    }

    public enum TooltipTextAlignment {
        LEFT,
        RIGHT,
        CENTERED
    }

}

==> sources_fancymenu_fabric_3.8.1_MC_1.21.10.jar: de/keksuccino/fancymenu/util/rendering/ui/tooltip/Tooltip.java <==
package de.keksuccino.fancymenu.util.rendering.ui.tooltip;

import com.mojang.blaze3d.systems.RenderSystem;
import de.keksuccino.fancymenu.FancyMenu;
import de.keksuccino.fancymenu.util.rendering.DrawableColor;
import de.keksuccino.fancymenu.util.rendering.RenderingUtils;
import de.keksuccino.fancymenu.util.rendering.text.TextFormattingUtils;
import de.keksuccino.fancymenu.util.rendering.ui.UIBase;
import de.keksuccino.fancymenu.util.resource.resources.texture.ITexture;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import net.minecraft.class_10799;
import net.minecraft.class_2561;
import net.minecraft.class_2583;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_327;
import net.minecraft.class_332;
import net.minecraft.class_4068;
import net.minecraft.class_437;
import net.minecraft.class_5250;
import net.minecraft.class_5348;
import net.minecraft.class_8002;

/**
 * A tooltip that gets rendered at the mouse position by default.<br>
 * It is possible to set a custom X and Y position to not render it at the mouse position.
 **/
@SuppressWarnings("unused")
public class Tooltip implements class_4068 {

    private static final Logger LOGGER = LogManager.getLogger();

    protected class_327 font = class_310.method_1551().field_1772;
    protected List<class_2561> textLines = new ArrayList<>();
    protected int width = 0;
    protected int height = 0;
    protected int aspectWidth = -1;
    protected int aspectHeight = -1;
    protected Integer x = null;
    protected Integer y = null;
    protected int textBorderSize = 5;
    protected int mouseOffset = 12;
    protected ITexture backgroundTexture = null;
    protected DrawableColor backgroundColor = null;
    protected DrawableColor borderColor = null;
    protected boolean vanillaLike = true;
    protected boolean keepBackgroundAspectRatio = true;
    protected boolean textShadow = true;
    protected TooltipTextAlignment textAlignment = TooltipTextAlignment.LEFT;
    protected DrawableColor textBaseColor = null;
    protected Float scale = null;

    @NotNull
    public static Tooltip empty() {
        return new Tooltip();
    }

    @NotNull
    public static Tooltip of(String... tooltip) {
        Tooltip t = new Tooltip();
        if (tooltip != null) {
            t.setTooltipText(tooltip);
        }
        return t;
    }

    @NotNull
    public static Tooltip of(class_2561... tooltip) {
        Tooltip t = new Tooltip();
        if (tooltip != null) {
            t.setTooltipText(tooltip);
        }
        return t;
    }

    @Override
    public void method_25394(@NotNull class_332 graphics, int mouseX, int mouseY, float partial) {

        if (RenderingUtils.isTooltipRenderingBlocked()) return;

        class_437 s = class_310.method_1551().field_1755;
        if (!this.isEmpty() && (s != null)) {

            this.updateAspectRatio();

            int x = this.calculateX(s, mouseX);
            int y = this.calculateY(s, mouseY);

            graphics.method_51448().pushMatrix();

            float scale = 1.0F;
            if (this.scale != null) {
                scale = UIBase.calculateFixedScale(this.scale);
                graphics.method_51448().scale(scale, scale);
            }
            graphics.method_51448().translate(0.0F, 0.0F);

            this.renderBackground(graphics, x, y);
            this.renderTextLines(graphics, x, y);

            graphics.method_51448().popMatrix();

        }
    }

    protected void renderTextLines(class_332 graphics, int x, int y) {
        int yLine = y;
        for (class_2561 c : this.textLines) {
            int w = this.font.method_27525(c);
            int x2 = x + this.textBorderSize;
            int y2 = yLine + this.textBorderSize;
            if (this.textAlignment == TooltipTextAlignment.RIGHT) {
                int diff = Math.max(0, (x + this.getWidth() - this.textBorderSize) - (x2 + w));
                x2 += diff;
            }
            if (this.textAlignment == TooltipTextAlignment.CENTERED) {
                x2 = x + Math.max(0, (this.getWidth() / 2) - (w / 2));
            }
            graphics.method_51439(this.font, c, x2, y2, (this.textBaseColor != null) ? this.textBaseColor.getColorInt() : -1, this.hasTextShadow());
            yLine += this.font.field_2000 + 2;
        }
    }

    protected void renderBackground(class_332 graphics, int x, int y) {
        if (this.vanillaLike || ((this.backgroundTexture == null) && (this.backgroundColor == null))) {
            this.renderVanillaLikeBackground(graphics, x, y, this.getWidth(), this.getHeight());
        } else if (this.backgroundTexture != null) {
            class_2960 loc = this.backgroundTexture.getResourceLocation();
            if (loc != null) {
                graphics.method_25290(class_10799.field_56883, loc, x, y, 0.0F, 0.0F, this.getWidth(), this.getHeight(), this.getWidth(), this.getHeight());
            }
        } else {
            if (this.borderColor != null) {
                //BACKGROUND
                graphics.method_25294(x + 1, y + 1, x + this.getWidth() - 1, y + this.getHeight() - 1, this.backgroundColor.getColorInt());
                //TOP
                graphics.method_25294(x + 1, y, x + this.getWidth() - 1, y + 1, this.borderColor.getColorInt());
                //BOTTOM
                graphics.method_25294(x + 1, y + this.getHeight() - 1, x + this.getWidth() - 1, y + this.getHeight(), this.borderColor.getColorInt());
                //LEFT
                graphics.method_25294(x, y, x + 1, y + this.getHeight(), this.borderColor.getColorInt());
                //RIGHT
                graphics.method_25294(x + this.getWidth() - 1, y, x + this.getWidth(), y + this.getHeight(), this.borderColor.getColorInt());
            } else {
                graphics.method_25294(x, y, x + this.getWidth(), y + this.getHeight(), this.backgroundColor.getColorInt());
            }
        }
    }

    protected void renderVanillaLikeBackground(class_332 graphics, int x, int y, int width, int height) {

        graphics.method_51448().pushMatrix();

        class_8002.method_47946(graphics, x, y, width, height, null);

        graphics.method_51448().popMatrix();

    }

    protected int calculateX(class_437 screen, int mouseX) {
        float scale = 1;
        if (this.scale != null) {
            scale = UIBase.calculateFixedScale(this.scale);
        }
        if (this.x != null) {
            mouseX = this.x;
        }
        int width = this.getWidth();
        if (this.vanillaLike) {
            width += 4;
        }
        int scaledWidth = (int)((float)width * scale);
        int scaledMouseX = (int)((float)mouseX / scale) + this.mouseOffset;
        int scaledScreenWidth = (int)((float)screen.field_22789 / scale);
        int x = (mouseX + this.mouseOffset);
        if ((x + scaledWidth) > screen.field_22789) {
            int offset = (x + scaledWidth) - screen.field_22789;
            x = x - offset;
        }
        return (int)((float)x / scale);
    }

    protected int calculateY(class_437 screen, int mouseY) {
        float scale = 1;
        if (this.scale != null) {
            scale = UIBase.calculateFixedScale(this.scale);
        }
        if (this.y != null) {
            mouseY = this.y;
        }
        int height = this.getHeight();
        if (this.vanillaLike) {
            height += 4;
        }
        int scaledHeight = (int)((float)height * scale);
        int scaledMouseY = (int)((float)mouseY / scale) + this.mouseOffset;
        int scaledScreenHeight = (int)((float)screen.field_22790 / scale);
        int y = (mouseY + this.mouseOffset);
        if ((y + scaledHeight) > screen.field_22790) {
            int offset = (y + scaledHeight) - screen.field_22790;
            y = y - offset;
        }
        return (int)((float)y / scale);
    }

    protected int getWidth() {
        if (this.aspectWidth != -1) {
            return this.aspectWidth;
        }
        return this.width;
    }

    protected int getHeight() {
        if (this.aspectHeight != -1) {
            return this.aspectHeight;
        }
        return this.height;
    }

    protected void updateAspectRatio() {
        if (!this.keepBackgroundAspectRatio || (this.backgroundTexture == null)) {
            this.aspectWidth = -1;
            this.aspectHeight = -1;
            return;
        }
        int[] size = this.backgroundTexture.getAspectRatio().getAspectRatioSizeByMinimumSize(this.width, this.height);
        this.aspectWidth = size[0];
        this.aspectHeight = size[1];
    }

    protected void updateSize() {
        int w = 0;
        int h = 0;
        for (class_2561 c : this.textLines) {
            int wl = this.font.method_27525(c);
            if (wl > w) {
                w = wl;
            }
            h += this.font.field_2000 + 2;
        }
        if (h > 0) {
            h -= 2;
        }
        this.width = w + (this.textBorderSize * 2);
        this.height = h + (this.textBorderSize * 2);
    }

    public boolean isEmpty() {
        return this.textLines.isEmpty();
    }

    public Tooltip setTooltipText(String... lines) {
        List<class_2561> l = new ArrayList<>();
        if (lines != null) {
            for (String s : lines) {
                l.add(class_2561.method_43470(s));
            }
        }
        return this.setTooltipText(l);
    }

    public Tooltip setTooltipText(class_2561... lines) {
        return this.setTooltipText((lines != null) ? Arrays.asList(lines) : null);
    }

    public Tooltip setTooltipText(List<class_2561> lines) {
        class_5250 merged = class_2561.method_43473();
        if (lines != null) {
            boolean b = true;
            for (class_2561 c : lines) {
                if (!b) merged.method_27693("\n");
                merged.method_10852(c);
                b = false;
            }
        }
        this.textLines = new ArrayList<>();
        for (class_5348 s : class_310.method_1551().field_1772.method_27527().method_27495(merged, 300, class_2583.field_24360)) {
            this.textLines.add(TextFormattingUtils.convertFormattedTextToComponent(s));
        }
        this.updateSize();
        return this;
    }

    /** Returns a COPY of the tooltip list. **/
    public List<class_2561> getTooltip() {
        return new ArrayList<>(this.textLines);
    }

    public Tooltip setTextBorderSize(int size) {
        this.textBorderSize = size;
        this.updateSize();
        return this;
    }

    public int getTextBorderSize() {
        return this.textBorderSize;
    }

    public Tooltip setMouseOffset(int offset) {
        this.mouseOffset = offset;
        return this;
    }

    public int getMouseOffset() {
        return this.mouseOffset;
    }

    public Tooltip setBackgroundTexture(@NotNull ITexture texture) {
        this.backgroundTexture = texture;
        this.backgroundColor = null;
        this.vanillaLike = false;
        return this;
    }

    @Nullable
    public ITexture getBackgroundTexture() {
        return this.backgroundTexture;
    }

    public Tooltip setBackgroundColor(@NotNull DrawableColor backgroundColor, @Nullable DrawableColor borderColor) {
        this.backgroundColor = backgroundColor;
        this.borderColor = borderColor;
        this.backgroundTexture = null;
        this.vanillaLike = false;
        return this;
    }

    public Tooltip setDefaultStyle() {
        this.setTextShadow(FancyMenu.getOptions().enableUiTextShadow.getValue());
        this.setTextBaseColor(UIBase.getUIColorTheme().element_label_color_normal);
        return this.setBackgroundColor(UIBase.getUIColorTheme().element_background_color_normal, UIBase.getUIColorTheme().element_border_color_normal);
    }

    @Nullable
    public DrawableColor getBackgroundColor() {
        return this.backgroundColor;
    }

    @Nullable
    public DrawableColor getBorderColor() {
        return this.borderColor;
    }

    public Tooltip setVanillaLike(boolean vanillaLike) {
        this.vanillaLike = vanillaLike;
        this.backgroundColor = null;
        this.backgroundTexture = null;
        return this;
    }

    public boolean isVanillaLike() {
        return vanillaLike;
    }

    public Tooltip setKeepBackgroundAspectRatio(boolean keepBackgroundAspectRatio) {
        this.keepBackgroundAspectRatio = keepBackgroundAspectRatio;
        return this;
    }

    public boolean keepBackgroundAspectRatio() {
        return keepBackgroundAspectRatio;
    }

    public Tooltip setTextShadow(boolean textShadow) {
        this.textShadow = textShadow;
        return this;
    }

    public boolean hasTextShadow() {
        return textShadow;
    }

    public Tooltip setTextAlignment(TooltipTextAlignment textAlignment) {
        this.textAlignment = textAlignment;
        return this;
    }

    public TooltipTextAlignment getTextAlignment() {
        return textAlignment;
    }

    public Tooltip setTextBaseColor(@Nullable DrawableColor textBaseColor) {
        this.textBaseColor = textBaseColor;
        return this;
    }

    @Nullable
    public DrawableColor getTextBaseColor() {
        return textBaseColor;
    }

    public Tooltip setFont(class_327 font) {
        this.font = (font != null) ? font : class_310.method_1551().field_1772;
        this.updateSize();
        return this;
    }

    public class_327 getFont() {
        return font;
    }

    public Tooltip copyStyleOf(Tooltip tooltip) {
        this.borderColor = tooltip.borderColor;
        this.backgroundColor = tooltip.backgroundColor;
        this.backgroundTexture = tooltip.backgroundTexture;
        this.keepBackgroundAspectRatio = tooltip.keepBackgroundAspectRatio;
        this.vanillaLike = tooltip.vanillaLike;
        this.textBorderSize = tooltip.textBorderSize;
        this.textAlignment = tooltip.textAlignment;
        this.textShadow = tooltip.textShadow;
        this.textBaseColor = tooltip.textBaseColor;
        this.font = tooltip.font;
        return this;
    }

    public Tooltip setCustomX(@Nullable Integer x) {
        this.x = x;
        return this;
    }

    @Nullable
    public Integer getCustomX() {
        return x;
    }

    public Tooltip setCustomY(@Nullable Integer y) {
        this.y = y;
        return this;
    }

    @Nullable
    public Integer getCustomY() {
        return y;
    }

    @Nullable
    public Float getScale() {
        return this.scale;
    }

    public Tooltip setScale(@Nullable Float scale) {
        this.scale = scale;
        return this;
    }

    public enum TooltipTextAlignment {
        LEFT,
        RIGHT,
        CENTERED
    }

}
